// GBAlib.h

#ifndef GBALIB_H
#define GBALIB_H

#define ADR_BGTIL0   *(u32*)0x6000000
#define ADR_BGTIL1   *(u32*)0x6004000
#define ADR_BGTIL2   *(u32*)0x6008000
#define ADR_BGTIL3   *(u32*)0x600C000

#define ADR_BGMAP0   *(u32*)0x6000000
#define ADR_BGMAP1   *(u32*)0x6000800
#define ADR_BGMAP2   *(u32*)0x6001000
#define ADR_BGMAP3   *(u32*)0x6001800
#define ADR_BGMAP4   *(u32*)0x6002000
#define ADR_BGMAP5   *(u32*)0x6002800
#define ADR_BGMAP6   *(u32*)0x6003000
#define ADR_BGMAP7   *(u32*)0x6003800

#define ADR_SRAM     (u8*)0xE000000

//BG
void BGSetPal(u32);
void BGSetChr(u8 No,u32 img,u16 size);
void BGSetMap(u8 BG, u16 x,u16 y,u16 palno,u16 chr);
void BGDrawStr(u8 BG,u8 x,u8 y,u8* str);
void BGShowNumber(u8 BG, u8, u8, u32, u8);

//Mode3
void Mode3PutPixel(u8 ,u8 ,u16 );
void Mode3DrawImage(u8 cx,u8 cy,u16* image);
void Mode3DrawBox(u8 ,u8 ,u8 ,u8 ,u16);

void Mode3SetBitMapFont(u8 *p, u16 cx, u16 cy, u16 cnt);
void Mode3DrawStr(u8 ,u8 ,u8*);
void Mode3DrawChar(u8 x,u8 y,u16 chr);
void Mode3DrawNumber(u8, u8, u32, u8);

void Mode3SetSJISFont(u32 code, u16 cnt, u32 offset, u8 cx, u8 cy);


//Sprite
void SpriteInitialize();
void SpriteMove(u16 num, s16 x, s16 y);
void SpriteSetSize(u16 num,u16 size,u16 form,u16 color);
void SpriteChange(u16 num, u16 ch);
void SpriteSetChr(u16 *img,u32 size);
void SpriteSetPalNo(u16 num, u16 palno);
void SpriteSetPal16(u16 *pal);
void SpriteSetPal256(u16 *pal);


//Sound
void SoundInitialize();
void SoundSetData(s8 *buf, u32 wave_size);
void SoundStart();
void SoundStop();

//fobN
void dprint(u8 *sz);
int  dprintf(char *fmt, ...);

//DMA]
void DMA3MemcpyFIX16(u32, u32, u16);
void DMA3MemcpyINC16(u32, u32, u16);
void DMA3MemcpyINC32(u32, u32, u16);

//
void *_malloc(u32 size);
void _mfree(void *ptr);

//̑
void WaitForVsync(void);

//exttfs
void exttfs_init(void);
void exttfs_SetManualAddress(u8 *data);
u8  *exttfs_GetDataAddr(void);
void exttfs_SkipData(u32 size);
u8   exttfs_readu8(void);
u16  exttfs_readu16(void);
u32  exttfs_readu32(void);
u8 *exttfs_fopenLZ77(u32 offset, u32 size);
u8 *exttfs_GetDataOffset(u32 offset);

//
void Set1block1bit(u8 font,int x1,int y1,u16 color);

void Mode3PutAscii(u8 x, u8 y, u16 num, u16 col);
void Mode3PutSJIS(u8 x, u8 y, u8* chr, u16 col);
void Mode3PutChar(u8 x, u8 y, u16 num, u16 col);

void Mode3Print(u8 x,u8 y,u8* texts,u16 color);

void Mode3PrintLength(u16* screen,int x,int y,u8* texts,u16 color,int length);
void Mode3PrintNum(u16* screen,int x,int y,int num,u16 color);



typedef struct{
  u8  *offset;
  u16 size;
  u16 cx;
  u16 cy;
  u16 cnt;
} ST_FONTBITMAP;

typedef struct{
  u16 *Code;
  u8  *Offset;
  u16 size;
  u16 cx;
  u16 cy;
  u16 cnt;
} ST_FONTSJIS;

extern ST_FONTBITMAP FONTBITMAP;
extern ST_FONTSJIS   FONTSJIS;

extern int wavecnt;
extern s8  *wavedata;
#endif
